
__version__ = "0.4.3"
__version_vcs_info__ = {
    'branch': 'master',
    'revid': '9f0e45e',
    'revno': 319}
__version_user_email__ = "skalkoto@grnet.gr"
__version_user_name__ = "Nikos Skalkotos"
